<?php declare(strict_types=1);

namespace Everyware\Plugin\GoogleAnalytics;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\Support\Str;

/**
 * Class Cron
 * @package Everyware\Plugin\GoogleAnalytics
 */
class Cron
{
    /**
     * @var string
     */
    public const INTERVAL_PREFIX = 'ew_';
    /**
     * @var array
     */
    private static $schedules;

    /**
     * Setup an array of custom intervals to be used in cron
     *
     * @since 0.1
     * @uses  add_filter()
     * @return void
     */
    public static function setupIntervals(): void
    {
        add_filter('cron_schedules', function ($schedules) {
            return static::$schedules = static::addDefaultSchedules($schedules);
        });
    }

    private static function addDefaultSchedules($schedules): array
    {
        $intervals = [
            60 => 'Every minute',
            300 => 'Every five minutes',
            900 => 'Every fifteen minutes',
            1800 => 'Every thirty minutes',
            2 * DAY_IN_SECONDS => 'Every two days',
            WEEK_IN_SECONDS => 'Once weekly'
        ];

        foreach ($intervals as $interval => $description) {
            $schedules[static::INTERVAL_PREFIX . Str::snake($description)] = [
                'interval' => $interval,
                'display' => $description
            ];
        }

        return $schedules;
    }

    /**
     * Register a cron
     *
     * @param string   $name
     * @param callable $callback
     * @param string   $interval
     *
     * @uses  add_action()
     * @uses  wp_next_scheduled()
     * @uses  wp_schedule_event()
     * @since 0.1
     * @return void
     */
    public static function register($name, callable $callback, $interval): void
    {
        // Only register if valid interval
        if (array_key_exists($interval, static::getIntervals())) {
            add_action($name, $callback);

            if ( ! wp_next_scheduled($name)) {
                wp_schedule_event(time(), $interval, $name);
            }
        }
    }

    /**
     * Get available intervals with Everyware's namespaces
     *
     * @uses  wp_get_schedules()
     * @since 0.1
     * @return array
     */
    public static function getIntervals(): array
    {
        $filtered_arr = array_filter(wp_get_schedules(), function ($key) {
            return Str::startsWith($key, static::INTERVAL_PREFIX) || Str::is('hourly', $key) || Str::is('twicedaily',
                    $key) || Str::is('daily', $key);
        }, ARRAY_FILTER_USE_KEY);

        return Arr::sort($filtered_arr, function ($value) {
            return strtolower($value['interval']);
        });
    }
}
