<?php declare(strict_types=1);

/*
 * Plugin Name: Most Read - Google Analytics
 * Description: A list of articles fetched from Google Analytics. Has option to add headline, link to section and provide custom view id.
 * Version: 1.0.0
 * Author: Infomaker Scandinavia AB
 * Author URI: https://infomaker.se
 */

namespace Everyware\Plugin\GoogleAnalytics;

use Infomaker\Everyware\Base\Pages;
use Infomaker\Everyware\Base\Templates;
use Infomaker\Everyware\Base\Widget;
use Infomaker\Everyware\Twig\View;

class MostReadWidget extends Widget
{

    /**
     * Twig-template to use
     *
     * @var string
     */
    protected $template_path = '@plugins/google-analytics/src/templates/widget/most-read';

    /** Contains the default template to use when rendering articles in the list
     *
     * @var string
     */
    private $defaultTemplate = 'ocarticle-line.php';

    /**
     * Contains the global settings for all ads
     *
     * @var Settings
     */
    protected static $settings;

    /**
     * @var array
     */
    protected $fields = [
        'custom_view_id' => '',
        'headline' => '',
        'limit' => 1,
        'remove_list_elements' => false,
        'page' => '',
        'start' => 1,
        'template' => '',
    ];

    /**
     * MostReadWidget constructor.
     */
    public function __construct()
    {
        $this->setWidgetData(__FILE__);
        parent::__construct('ew-google-analytics-most-read');

        // Add global settings to all widgets
        if (static::$settings === null) {
            static::$settings = new Settings();
        }
    }

    /**
     * Fires when rendering the widget form
     *
     * @param array $view_data
     *
     * @return void
     */
    protected function edit($view_data): void
    {
        $view_data = \array_replace_recursive($view_data, [
            'pages' => Pages::getSelectData(),
            'templates' => Templates::getSelectData()
        ]);

        View::render($this->template_path . '/form', $view_data);
    }

    /**
     * Fires when rendering widget
     *
     * @param array $view_data
     */
    protected function show($view_data): void
    {
        $client = new GoogleAnalyticsClient(static::$settings);

        if ($view_data['custom_view_id'] !== '') {
            $client->customize([
                'view_id' => $view_data['custom_view_id']
            ]);
        }

        // Get real article data back from WP and OC
        $articles = $client->getArticlesFromPosition($view_data['start'], $view_data['limit']);

        $view_data['articles'] = $this->generateFromTemplate($articles, $view_data['template']);

        View::render($this->template_path . '/widget', $view_data);
    }

    /**
     * Generate from template
     *
     * @param array $oc_articles
     * @param       $template
     *
     * @return array
     */
    private function generateFromTemplate(array $oc_articles, $template): array
    {
        $chosenTemplate = ! empty($template) ? $template : $this->defaultTemplate;

        $templatePath = \locate_template([$chosenTemplate, '/templates/' . $chosenTemplate]);

        $articles = [];
        \ob_start();
        foreach ($oc_articles as $article) {
            \load_template( $templatePath, false );
//            include $templatePath;
            $articles[] = \ob_get_contents();
            \ob_clean();
        }
        \ob_end_clean();

        return $articles;
    }
}
